##############################################################################
#
# Copyright (c) 2007 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
$Id: widget.py 84455 2008-03-04 11:16:16Z jukart $
"""
__docformat__ = 'restructuredtext'

import simplejson
import urlparse, cgi, urllib
from xml.dom.minidom import parse, parseString

import zope.component

from zope.formlib import form
from zope.security.proxy import removeSecurityProxy
from zope.event import notify
from zope.traversing.browser import absoluteURL
from zope.traversing.interfaces import TraversalError
from zope.cachedescriptors.property import Lazy
from zope.publisher.browser import TestRequest

from zope.app.intid.interfaces import IIntIds
from zope.app.form.browser.textwidgets import TextWidget
from zope.app.component import hooks
from zope.app.form.browser.widget import renderElement
from zope.app.form.browser.textwidgets import BytesWidget
from zope.app.pagetemplate import ViewPageTemplateFile

from zc import resourcelibrary

from z3c.reference import interfaces

from lovely.relation.dataproperty import DataRelationship

from views import getEditorView, getOpenerView
from serialize import serializeRelation


class ViewReferenceWidget(TextWidget):
    """renders an "a" tag with the title and href attributes."""

    template = ViewPageTemplateFile('widget.pt')

    tag = u'input'
    type = u'text'
    cssClass = u'popupwindow'
    extra =  u'rel="window"'
    refTag = u'a'
    refTagOnClick=""
    _emptyReference = None
    referenceExplorerViewName = 'viewReferenceEditor.html'
    functionName = 'setDefaultReferenceInput'
    _missing = None

    def __init__(self, *args):
        resourcelibrary.need('z3c.reference.parent')
        super(ViewReferenceWidget, self).__init__(*args)

    @property
    def referenceEditorURL(self):
        """Returns the reference explorer url."""
        return absoluteURL(self.context.context, self.request) + '/%s?%s' % (
            self.referenceExplorerViewName,
            urllib.urlencode({'settingName' : self.context.settingName,
                              'target' : self.targetValue,
                              'name': self.name,
                              'formdata': self.formDataValue,
                              }))

    @property
    def formDataValue(self):
        ref = self._getCurrentValue()
        if ref is None or ref.target is None:
            return ''
        return serializeRelation(ref, self.request, self.context.settingName)

    @property
    def viewValue(self):
        """Returns the reference view string."""
        current = self._getCurrentValue()
        if current and current.view:
            return current.view or u''
        else:
            return u''

    @property
    def targetValue(self):
        """Returns the target intid."""
        target = u''
        current = self._getCurrentValue()
        if current and current.target:
            intIds = zope.component.getUtility(IIntIds)
            target = intIds.getId(current.target)
        return target

    @property
    def refIdName(self):
        return self.name +  u'.refId'

    @property
    def refIdValue(self):
        """Returns the target intid."""
        refId = u''
        current = self._getCurrentValue()
        if current:
            intIds = zope.component.getUtility(IIntIds)
            refId = intIds.queryId(current)
            if refId is None:
                refId = u''
        return refId

    def __call__(self):
        resourcelibrary.need('z3c.reference.parent')
        if self._renderedValueSet():
            ref = self._data
        else:
            ref = self.context.default
        if not ref:
            try:
                ref = self.context.get(self.context.context)
            except:
                ref = None
        openerView = getOpenerView(ref, self.request,
                                   self.context.settingName)
        openerView.prefix = self.name
        contents = openerView()

        removeName = self.name + '.remove'
        targetName = self.name + '.target'
        formDataName = self.name + '.formData'
        settingNameFieldName = self.name + '.settingName'
        functionFieldName=self.name + '.function'
        funcInput = renderElement(u'input',
                                   type='hidden',
                                   name=functionFieldName,
                                   id=functionFieldName,
                                   value=self.functionName,
                                   extra=self.extra)
        intidInput = renderElement(u'input',
                                   type='hidden',
                                   name=targetName,
                                   id=targetName,
                                   value=self.targetValue,
                                   extra=self.extra)
        refIdInput = renderElement(u'input',
                                   type='hidden',
                                   name=self.refIdName,
                                   id=self.refIdName,
                                   value=self.refIdValue,
                                   extra=self.extra)
        formDataInput = renderElement(u'input',
                                      type='hidden',
                                      name=formDataName,
                                      id=formDataName,
                                      value=self.formDataValue,
                                      extra=self.extra)
        settingNameInput = renderElement(u'input',
                                      type='hidden',
                                      name=settingNameFieldName,
                                      id=settingNameFieldName,
                                      value = self.context.settingName,
                                      extra=self.extra)
        linkTag = renderElement(self.refTag,
                        href = self.referenceEditorURL,
                        name=self.name,
                        id=self.name + '.tag',
                        onclick=self.refTagOnClick,
                        cssClass = self.cssClass,
                        contents=contents,
                        style=self.style,
                        extra = 'rel="height:550,width:800,status:0,scrollbars:1,resizable:1"',
                        )
        if not self.context.required:
            removeButton = renderElement(u'input',
                                         type='checkbox',
                                         name=removeName,
                                         id=removeName,
                                         value=u'',
                                         )
        else:
            removeButton = None
        return self.template(removeButton=removeButton,
                             removeButtonName=removeName,
                             funcInput=funcInput,
                             linkTag=linkTag,
                             intidInput=intidInput,
                             formDataInput=formDataInput,
                             refIdInput=refIdInput,
                             settingNameInput=settingNameInput,
                             )

    def _toFormValue(self, value):
        if value == self.context.missing_value:
            return self._missing
        try:
            url = absoluteURL(value,self.request)
        except TypeError:
            return self._missing
        return url

    def hasInput(self):
        return self.name + '.target' in self.request.form

    def _getFormInput(self):
        value = self.request.form.get(self.name + '.target')
        if not value:
            return None
        return value

    def _toFieldValue(self, input):
        if input is None:
            return None
        if self.name + '.remove' in self.request.form:
            return None
        if input == self._missing:
            return self.context.missing_value
        intIds = zope.component.getUtility(IIntIds)
        # XXX this does not work with lists
        refId = self.request.form.get(self.refIdName)
        ref = None
        if refId:
            ref = intIds.getObject(int(refId))

        # form field ids
        formDataName = self.name + '.formData'
        targetName = self.name + '.target'

        # get target obj str
        intid = self.request.get(targetName)
        if intid is None:
            return self.context.missing_value
        try:
            obj = intIds.queryObject(int(intid))
        except ValueError:
            return self.context.missing_value
        if obj is None:
            return self.context.missing_value
        if ref is None:
            ref = DataRelationship(obj)
        else:
            ref.target = obj

        # apply the form data
        formData = self.request.get(formDataName)
        if not formData:
            return ref
        data = cgi.parse_qs(str(formData), keep_blank_values=True)
        for k, v in data.items():
            if type(v) is type([]) and len(v)==1:
                data[k] = v[0].decode('utf8')
        if 'form.view' in data:
            data['form.view'] = data['form.view'].replace('&amp;amp;','&amp;')

        data['form.actions.apply'] = u''
        r = TestRequest(form=data)
        klass = getEditorView(ref.target, self.request,
                              self.context.settingName).__class__
        view = klass(ref, r)
        view = ApplyForm(ref, r, view.form_fields)
        view.update()
        return ref


class ApplyForm(form.EditForm):

    def __init__(self, context, request, form_fields):
        self.form_fields = form_fields
        super(ApplyForm, self).__init__(context, request)


class ObjectReferenceWidget(ViewReferenceWidget):

    @Lazy
    def extra(self):
        iface = self.context.refSchema
        name = u'%s.%s' % (iface.__module__,iface.__name__)
        return u'z3c:explorerLink="@@explorer.html?link=1&schema=%s"' % name


class CropImageWidget(BytesWidget):
    """widget for cropping images"""

    type = u'hidden'

    template = ViewPageTemplateFile('crop-image-widget.pt')

    # the following properties must be set by the editor view
    cropWidth = 0
    cropHeight = 0
    ratioPresets = [{'name': 'None'}]

    @property
    def ajaxRatioPresets(self):
        result = simplejson.dumps(self.ratioPresets)
        return result.replace('"', '\\\'')

    def url(self):
        if interfaces.IViewReference.providedBy(self.context.context):
            return absoluteURL(self.context.context.__parent__.target,
                               self.request)
        else:
            return absoluteURL(self.context.context, self.request)

    def inputField(self):
        return super(CropImageWidget, self).__call__()

    def escapedName(self):
        return self.name.replace('.', r'\.')

    def __call__(self, *args, **kw):
        resourcelibrary.need('z3c.javascript.swfobject')
        return self.template(*args, **kw)

